import 'dart:math';

import 'package:flutter/material.dart';

class ScreenUtil {
  static const Size defaultSize = Size(360, 690);
  static late ScreenUtil _instance;

  late Size uiSize;

  late Orientation _orientation;

  late double _screenWidth;
  late double _screenHeight;
  late bool _minTextAdapt;
  late BuildContext? context;
  late bool _splitScreenMode;

  ScreenUtil._();

  factory ScreenUtil() {
    return _instance;
  }

  static void setContext(BuildContext context) {
    _instance.context = context;
  }

  static void init(
    BoxConstraints constraints, {
    BuildContext? context,
    Orientation orientation = Orientation.portrait,
    Size designSize = const Size(411, 820),
    bool splitScreenMode = false,
    bool minTextAdapt = false,
  }) {
    _instance = ScreenUtil._()
      ..uiSize = designSize
      .._splitScreenMode = splitScreenMode
      .._minTextAdapt = minTextAdapt
      .._orientation = orientation
      .._screenWidth = constraints.maxWidth
      .._screenHeight = constraints.maxHeight;
    if (context != null) setContext(context);
  }

  Orientation get orientation => _orientation;

  double get textScaleFactor =>
      context != null ? MediaQuery.of(context!).textScaleFactor : 1;

  double? get pixelRatio =>
      context != null ? MediaQuery.of(context!).devicePixelRatio : 1;

  double get screenWidth =>
      context != null ? MediaQuery.of(context!).size.width : _screenWidth;

  double get screenHeight =>
      context != null ? MediaQuery.of(context!).size.height : _screenHeight;

  double get statusBarHeight =>
      context == null ? 0 : MediaQuery.of(context!).padding.top;

  double get bottomBarHeight =>
      context == null ? 0 : MediaQuery.of(context!).padding.bottom;

  double get scaleWidth => screenWidth / uiSize.width;

  double get scaleHeight =>
      (_splitScreenMode ? max(screenHeight, 700) : screenHeight) /
      uiSize.height;

  double get scaleText =>
      _minTextAdapt ? min(scaleWidth, scaleHeight) : scaleWidth;

  double setWidth(num width) => width * scaleWidth;

  double setHeight(num height) => height * scaleHeight;

  double radius(num r) => r * min(scaleWidth, scaleHeight);

  double setSp(num fontSize) => fontSize * scaleText;

  Widget setVerticalSpacing(num height) => SizedBox(height: setHeight(height));

  Widget setVerticalSpacingFromWidth(num height) =>
      SizedBox(height: setWidth(height));

  Widget setHorizontalSpacing(num width) => SizedBox(width: setWidth(width));

  Widget setHorizontalSpacingRadius(num width) =>
      SizedBox(width: radius(width));

  Widget setVerticalSpacingRadius(num height) =>
      SizedBox(height: radius(height));
}
